@@ -231,10 +231,10 @@ const char *module_to_desc (GModule module);
 const char *module_to_head (GModule module);
 const char *module_to_id (GModule module);
 const char *module_to_label (GModule module);
+GAgents *load_host_agents (const char *addr);
 int get_start_end_parsing_dates (char **start, char **end, const char *f);
 int render_confdlg (Logs * logs, GSpinner * spinner);
 void close_win (WINDOW * w);
-GAgents *load_host_agents (const char *addr);
 void display_general (WINDOW * win, GHolder *h);
 void draw_header (WINDOW * win, const char *s, const char *fmt, int y, int x, int w, GColors * (*func) (void));
 void end_spinner (void);
@@ -245,11 +245,13 @@ void load_agent_list (WINDOW * main_win, char *addr);
 void load_help_popup (WINDOW * main_win);
 void load_schemes_win (WINDOW * main_win);
 void load_sort_win (WINDOW * main_win, GModule module, GSort * sort);
+void lock_spinner (void);
 void set_curses_spinner (GSpinner *spinner);
 void set_input_opts (void);
 void set_wbkgd (WINDOW *main_win, WINDOW *header_win);
 void term_size (WINDOW * main_win, int *main_win_height);
 void ui_spinner_create (GSpinner * spinner);
+void unlock_spinner (void);
 void update_active_module (WINDOW * header_win, GModule current);
 void update_header (WINDOW * header_win, int current);
 /* *INDENT-ON* */
